use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  JoinObjectPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::MultiStore::API::CloneMasterShop qw ( CloneMasterShop CreateCacheShop RegisterCacheShopAtSubShop );
use DE_EPAGES::MultiStore::API::Object::PagerVisibleProducts;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use Readonly;
use Test::More;

use strict;
Readonly my $ROOTPATH => 'Test';
my $ABSOLUTE_ROOTPATH = JoinObjectPath( [ '', $ROOTPATH ] );

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # make sure that the cartridge is installed
            unless ( ExistsCartridgeByPackage('DE_EPAGES::MultiStore') ) {
                plan skip_all => 'MultiStore cartridge not installed';
            }

            # make sure that the cartridge is installed
            unless ( ExistsCartridgeByPackage('DE_EPAGES::MultiStoreMaster') ) {
                plan skip_all => 'MultiStoreMaster cartridge not installed';
            }

            ok( Init(), 'init' );

            # tests
            eval { RunTests(); };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub RunTests {
    TestCreateCacheShop();
    TestLinkCategory();
    TestLinkProduct();
    TestPager();
}

sub TestLinkCategory {
    my $SubShop   = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestSubShop' ] ) );
    my $CacheShop = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestCacheShop' ] ) );

    my $AffiliateObject = $CacheShop->path('Categories/Jackets');
    eval { $AffiliateObject->cloneToSubShop( $SubShop->path('Categories') ); };
    if ( ExistsError() ) {
        my $Error = GetError();
        fail( 'Error: ' . $Error->longMessage );
        return;
    }

    my $SubShopCategory = $SubShop->path('Categories/Jackets');
    like( $SubShopCategory->getWithMaster('ImageSmall'), qr{Categories/Jackets/Jacket.gif}, 'ImageSmall' );
    is( scalar @{ $SubShopCategory->getWithMaster('VisibleProducts') }, 3, 'visible products' );
}

sub TestLinkProduct {
    my $SubShop   = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestSubShop' ] ) );
    my $CacheShop = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestCacheShop' ] ) );

    my $AffiliateObject = $CacheShop->path('Products/be_40401');
    eval { $AffiliateObject->cloneToSubShop( $SubShop->path('Categories') ); };
    if ( ExistsError() ) {
        my $Error = GetError();
        fail( 'Error: ' . $Error->longMessage );
        return;
    }
    my $SubShopProduct = $SubShop->path('Products/be_40401');
    is( scalar @{ $SubShopProduct->getWithMaster('SubProducts') }, 10, '10 subproducts' );
    $SubShopProduct->set({'Name' => 'SubShopProduct'}, 1);
    is($SubShopProduct->get('Name',1), 'SubShopProduct', 'product name (get)');
    is($SubShopProduct->getWithMaster('Name',1), 'SubShopProduct', 'product name (getWithMaster)');
}

sub TestCreateCacheShop {
    my $SubShop = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestSubShop' ] ) );
    my $CacheShop = CreateCacheShop( {
            'MasterStore' => GetTestSite(),
            'MasterShop'  => $ABSOLUTE_ROOTPATH . '/TestMasterShop',
            'CacheShop'   => $ABSOLUTE_ROOTPATH . '/TestCacheShop',
        },
        0
    );
    pass('create cache shop');
    CloneMasterShop($CacheShop);
    pass('clone master shop');
    RegisterCacheShopAtSubShop( $CacheShop, $SubShop );
    is( $SubShop->get('CacheShop')->id, $CacheShop->id, 'registered cache shop.' );
}

sub TestPager {
    my $CacheShop = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestCacheShop' ] ) );
    my $Rucksacks = $CacheShop->path('Categories/Rucksacks');
    my $Pager     = DE_EPAGES::MultiStore::API::Object::PagerVisibleProducts->new( {
            'Object'   => $Rucksacks,
            'ObjectID' => $Rucksacks->id,
            'PageSize' => 2,
            'OrderBy'  => 'Alias',
        }
    );

    $Pager->statistic();
    my $hParams = $Pager->page();
    my $Page    = $hParams->{'Pager'};
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 2;
    is( $Page->{'CountOfItems'}, 5, 'count of items' );
    is( $Page->{'CountOfPages'}, 3, 'count of pages' );
    is( $Page->{'FirstPage'},    1, 'first page' );
    is( $Page->{'NextPage'},     2, 'next page' );
    is( $Page->{'LastPage'},     3, 'last page' );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    BlockWarnings(
        sub {
            eval { LoadObjectByPath($ABSOLUTE_ROOTPATH)->delete if ExistsObjectByPath($ABSOLUTE_ROOTPATH); };
            Warning( GetError() ) if ( ExistsError() );
        }
    );
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ABSOLUTE_ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => $ROOTPATH } );
    ImportObjects( 'MasterShopComplete.xml', $ABSOLUTE_ROOTPATH );
    ImportObjects( 'SubShopEmpty.xml',       $ABSOLUTE_ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
