use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::MultiStore::API::Object qw ( SerializeObject DeserializeObject );
use DE_EPAGES::MultiStore::API::Object::RemoteObjectPlaceholder;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadRootObject );
use Scalar::Util qw ( blessed );
use Test::More;
BEGIN {
    *MultiObj:: = *DE_EPAGES::MultiStore::API::Object::;
}

## no critic (Subroutines::ProtectPrivateSubs)
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # tests
            TestSimple();
            TestSerializeAPI();
            TestDeserializeAPI();
            done_testing();
        }
    );
}

sub TestSimple {

    my $hInfo = {
        '/Classes/Object' => DE_EPAGES::MultiStore::API::Object::RemoteObjectPlaceholder->new(
            'Object' => LoadObjectByPath('/Classes/Object')
        ),
        '/Classes/Shop' => DE_EPAGES::MultiStore::API::Object::RemoteObjectPlaceholder->new(
            'Object' => LoadObjectByPath('/Classes/Shop')
        ),
    };

    while ( my ( $key, $value ) = each %$hInfo ) {
        my $rInfo = \$hInfo->{$key};
        $$rInfo = $value->doDeserialize() if ( blessed $value && $value->can('doSerialize') );
        is( $value->serializedObject(), $key, 'serialized: ' . $key );
        is( $hInfo->{$key}->pathString, $key, 'objectpath: ' . $key );
    }
}

sub TestSerializeAPI {
    _serializeSimple();
    _serialize0();
    _serialize1();
    _serialize2();
    _serialize3();
}

sub TestDeserializeAPI {
    my $hReturn = MultiObj::_ModifyReturn( {
            'a' =>
              DE_EPAGES::MultiStore::API::Object::RemoteObjectPlaceholder->new( 'serializedObject' => '/Classes/Shop' ),
        },
        sub { return DeserializeObject( shift, LoadRootObject() ) },
        1
    );
    isa_ok( $hReturn->{'a'}, 'DE_EPAGES::Object::API::Object::Class', 'deserialized object' );
}

sub _serializeSimple {
    is( MultiObj::_ModifyReturn( '', sub { return SerializeObject(shift) }, 1 ), '', 'empty string' );
    is_deeply( MultiObj::_ModifyReturn( {}, sub { return SerializeObject(shift) }, 1 ), {}, 'empty hash' );
    is_deeply(
        MultiObj::_ModifyReturn( { 'a' => 'b' }, sub { return SerializeObject(shift) }, 1 ),
        { 'a' => 'b' },
        'simple hash'
    );
    is_deeply(
        MultiObj::_ModifyReturn( [ 'a', 'b', 'c' ], sub { return SerializeObject(shift) }, 1 ),
        [ 'a', 'b', 'c' ],
        'simple array ref'
    );
}

sub _serialize0 {
        my $hReturn = MultiObj::_ModifyReturn( { 'a' => LoadObjectByPath('/Classes/Object'), 'b' => 'test' },
        sub { return SerializeObject(shift) }, 1 );
    isa_ok(
        $hReturn->{'a'},
        'DE_EPAGES::MultiStore::API::Object::RemoteObjectPlaceholder',
        'serialize hash object element.'
    );
    is( $hReturn->{'a'}->serializedObject(), '/Classes/Object', 'serialized object path.' );
}

sub _serialize1 {
    my $hReturn = MultiObj::_ModifyReturn( {
            'a' => [
                LoadObjectByPath('/Classes/Object'), LoadObjectByPath('/Classes/Object'),
                LoadObjectByPath('/Classes/Object')
            ],
            'b' => 'test'
        },
        sub { return SerializeObject(shift) },
        1
    );

    is( scalar @{ $hReturn->{'a'} }, 3, 'serialize hash array containing objects.' );
    isa_ok(
        $hReturn->{'a'}->[0],
        'DE_EPAGES::MultiStore::API::Object::RemoteObjectPlaceholder',
        'serialize hash array object element.'
    );
}

sub _serialize2 {
    my $hReturn = MultiObj::_ModifyReturn( {
            'a' => {
                'Key'    => LoadObjectByPath('/Classes/Object'),
                'Values' => [ LoadObjectByPath('/Classes/Object'), LoadObjectByPath('/Classes/Object'), ],
            }
        },
        sub { return SerializeObject(shift) },
        2
    );

    isa_ok(
        $hReturn->{'a'}->{'Key'},
        'DE_EPAGES::MultiStore::API::Object::RemoteObjectPlaceholder',
        'serialize hash of hash.'
    );
    isa_ok(
        $hReturn->{'a'}->{'Values'}->[0],
        'DE_EPAGES::MultiStore::API::Object::RemoteObjectPlaceholder',
        'serialize array in hash of hash.'
    );
    
}

sub _serialize3 {
    my $hReturn = MultiObj::_ModifyReturn( {
            'UserDefinedAttributeValues' => [ {
                    'Key'   => LoadObjectByPath('/Classes/Object'),
                    'Value' => LoadObjectByPath('/Classes/Object'),
                }
            ],
        },
        sub { return SerializeObject(shift) },
        2
    );

    isa_ok(
        $hReturn->{'UserDefinedAttributeValues'}->[0]->{'Key'},
        'DE_EPAGES::MultiStore::API::Object::RemoteObjectPlaceholder',
        'key serialize array of hashes.'
    );
    isa_ok(
        $hReturn->{'UserDefinedAttributeValues'}->[0]->{'Value'},
        'DE_EPAGES::MultiStore::API::Object::RemoteObjectPlaceholder',
        'value serialize array of hashes.'
    );
}

RunScript( Sub => \&Main );

1;
