use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadObjectByPath
  LoadRootObject
  LoadClassByAlias
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::MultiStoreMaster::API::AffiliateVisibility qw ( AddShopsToAffiliateVisibility RemoveShopsFromAffiliateVisibility IsAffiliateVisible );
use Readonly;
use Test::More;

Readonly my $ROOTPATH => '/Test';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                TestAffiliateVisibilityCategories();
                TestAffiliateVisibilityProducts();
                TestAffiliateShopVisibilityCategories();
                TestAffiliateShopVisibilityProducts();
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub TestAffiliateVisibilityCategories {
    my $Shop    = LoadObjectByPath('/Test/TestMasterShop');
    my $RootCat = $Shop->get('Categories');
    ok(!$RootCat->get('IsAffiliateVisible'), 'Categories: IsAffiliateVisible=undef' );
    my $Tents = $RootCat->child('Tents');
    is( $Tents->get('IsAffiliateVisible'), 1, 'Tents: IsAffiliateVisible=1' );
    my $FamilyTents = $Tents->child('FamilyTents');
    is( $FamilyTents->get('IsAffiliateVisible'), 1, 'FamilyTents: IsAffiliateVisible=1' );
    $Tents->set( { 'IsAffiliateVisible' => 0 } );
    is( $Tents->get('IsAffiliateVisible'),       0, 'Tents: IsAffiliateVisible=0' );
    is( $FamilyTents->get('IsAffiliateVisible'), 0, 'FamilyTents: IsAffiliateVisible=0 (recursive)' );
    my $TestTents = $FamilyTents->insertChild( 'Category', { 'Alias' => 'TestTents', } );
    is( $TestTents->get('IsAffiliateVisible'), 0, 'TestTents: IsAffiliateVisible=0 (recursive two levels)' );
    $FamilyTents->set( { 'IsAffiliateVisible' => 1 } );
    is( $TestTents->get('IsAffiliateVisible'), 1, 'TestTents: IsAffiliateVisible=1 (recursive one level)' );
    $FamilyTents->set( { 'IsAffiliateVisible' => undef } );
    is( $TestTents->get('IsAffiliateVisible'), 0, 'TestTents: IsAffiliateVisible=1 (recursive two levels again)' );

    # check the map directly
    my $AffiliateMapClass = LoadClassByAlias('AffiliateVisibilityMap');
    is( scalar @{ $AffiliateMapClass->getObjects( 'ByMasterShopContentObjectID', [ $TestTents->id ] ) },   0, 'TestTents: No MapObject' );
    is( scalar @{ $AffiliateMapClass->getObjects( 'ByMasterShopContentObjectID', [ $FamilyTents->id ] ) }, 0, 'FamilyTents: No MapObject' );
    is( scalar @{ $AffiliateMapClass->getObjects( 'ByMasterShopContentObjectID', [ $Tents->id ] ) },       1, 'Tents: One MapObject' );
}

sub TestAffiliateVisibilityProducts {
    my $Shop     = LoadObjectByPath('/Test/TestMasterShop');
    my $Folder   = $Shop->child('Products');
    my $be_40401 = $Folder->child('be_40401');
    is( $be_40401->getWithParent('IsAffiliateVisible'), 1, 'be_40401: IsAffiliateVisible=1 (by default)' );
    my $be_4040104003 = $be_40401->child('SubProducts')->child('be_4040104003');
    is( $be_4040104003->getWithParent('IsAffiliateVisible'), 1, 'be_4040104003: IsAffiliateVisible=1 (getWithParent)' );
    $be_40401->set( { 'IsAffiliateVisible' => 0 } );
    is( $be_40401->getWithParent('IsAffiliateVisible'),      0, 'be_40401: IsAffiliateVisible=0 (set)' );
    is( $be_4040104003->getWithParent('IsAffiliateVisible'), 0, 'be_4040104003: IsAffiliateVisible=0 (getWithParent)' );

    # check the map directly
    my $AffiliateMapClass = LoadClassByAlias('AffiliateVisibilityMap');
    is( scalar @{ $AffiliateMapClass->getObjects( 'ByMasterShopContentObjectID', [ $be_40401->id ] ) },      1, 'be_40401: One MapObject' );
    is( scalar @{ $AffiliateMapClass->getObjects( 'ByMasterShopContentObjectID', [ $be_4040104003->id ] ) }, 0, 'be_4040104003: No MapObject' );

}

sub TestAffiliateShopVisibilityCategories {
    my $Shop    = LoadObjectByPath('/Test/TestMasterShop');
    my $RootCat = $Shop->get('Categories');
    my $Tents   = $RootCat->child('Tents');
    $Tents->set( { 'IsAffiliateVisible' => 1 } );
    $Tents->set( { 'HasSelectiveAffiliateVisibility' => 1 } );
    is( $Tents->get('HasSelectiveAffiliateVisibility'), 1, 'Tents: HasSelectiveAffiliateVisibility=1' );
    my $FamilyTents = $Tents->child('FamilyTents');
    is( $FamilyTents->get('HasSelectiveAffiliateVisibility'), 1, 'FamilyTents: HasSelectiveAffiliateVisibility=1' );
    my $aShops = $Tents->get('AffiliateShopAliases');
    is( scalar @$aShops, 0, 'no affiliate shops' );
    AddShopsToAffiliateVisibility( $Tents, [ 'Shop1', 'Shop2' ] );
    $aShops = $Tents->get('AffiliateShopAliases');
    is( scalar @$aShops, 2, 'two affiliate shops' );
    ok( 'Shop1' ~~ @$aShops, 'shop1 in affiliate shops' );
    ok( 'Shop2' ~~ @$aShops, 'shop2 in affiliate shops' );

    RemoveShopsFromAffiliateVisibility( $Tents, ['Shop2'] );
    $aShops = $Tents->get('AffiliateShopAliases');
    is( scalar @$aShops, 1,       'one affiliate shop left' );
    is( $aShops->[0],    'Shop1', 'shop1 left' );

    ok( IsAffiliateVisible( $Tents,       'Shop1' ), 'Tents is AffiliateVisible in Shop1 (API)' );
    ok( IsAffiliateVisible( $FamilyTents, 'Shop1' ), 'FamilyTents is AffiliateVisible in Shop1 (API)' );
    ok( !IsAffiliateVisible( $Tents,       'Shop2' ), 'Tents isnt AffiliateVisible in Shop2 (API)' );
    ok( !IsAffiliateVisible( $FamilyTents, 'Shop2' ), 'FamilyTents isnt AffiliateVisible in Shop2 (API)' );
}

sub TestAffiliateShopVisibilityProducts {
    my $Shop             = LoadObjectByPath('/Test/TestMasterShop');
    my $ProductFolder    = $Shop->child('Products');
    my $Product_be_40401 = $ProductFolder->child('be_40401');
    $Product_be_40401->set( { 'IsAffiliateVisible' => 1 } );
    is( $Product_be_40401->get('IsAffiliateVisible'),              1, 'be_40401: IsAffiliateVisible' );
    is( $Product_be_40401->get('HasSelectiveAffiliateVisibility'), 0, 'be_40401: HasSelectiveAffiliateVisibility=0' );
    $Product_be_40401->set( { 'HasSelectiveAffiliateVisibility' => 1 } );
    is( $Product_be_40401->get('HasSelectiveAffiliateVisibility'), 1, 'be_40401: HasSelectiveAffiliateVisibility=1' );
    my ($Product_be_4040104003) = grep { $_->alias eq 'be_4040104003'} @{$Product_be_40401->get('SubProducts')};
    is( $Product_be_4040104003->getWithParent('IsAffiliateVisible'),              1, 'be_4040104003: IsAffiliateVisible=1' );
    is( $Product_be_4040104003->getWithParent('HasSelectiveAffiliateVisibility'), 1, 'be_4040104003: HasSelectiveAffiliateVisibility=1' );
    my $aShops = $Product_be_40401->get('AffiliateShopAliases');
    is( scalar @$aShops, 0, 'no affiliate shops' );
    AddShopsToAffiliateVisibility( $Product_be_40401, [ 'Shop1', 'Shop2' ] );
    $aShops = $Product_be_40401->get('AffiliateShopAliases');
    is( scalar @$aShops, 2, 'two affiliate shops' );
    ok( 'Shop1' ~~ @$aShops, 'shop1 in affiliate shops' );
    ok( 'Shop2' ~~ @$aShops, 'shop2 in affiliate shops' );

    RemoveShopsFromAffiliateVisibility( $Product_be_40401, ['Shop2'] );
    $aShops = $Product_be_40401->get('AffiliateShopAliases');
    is( scalar @$aShops, 1,       'one affiliate shop left' );
    is( $aShops->[0],    'Shop1', 'shop1 left' );

    ok( IsAffiliateVisible( $Product_be_40401,       'Shop1' ), 'be_40401 is AffiliateVisible in Shop1 (API)' );
    ok( IsAffiliateVisible( $Product_be_4040104003, 'Shop1' ), 'be_4040104003 is AffiliateVisible in Shop1 (API)' );
    ok( !IsAffiliateVisible( $Product_be_40401,       'Shop2' ), 'be_40401 isnt AffiliateVisible in Shop2 (API)' );
    ok( !IsAffiliateVisible( $Product_be_4040104003, 'Shop2' ), 'be_4040104003 isnt AffiliateVisible in Shop2 (API)' );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'TestShop.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
