use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use Readonly;
use Test::More;
use Test::Exception;
use DE_EPAGES::MultiStoreProvider::API::ShopRef qw (IsSubShopAssignedToMasterShop AssignSubShopToMasterShop UnassignSubShopFromMasterShop );

Readonly my $ROOTPATH => '/Test';

## no critic (ProhibitPathConcatenation)
sub Main {
    RunOnStore(
        'Store' => 'Site',
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                TestAssignSubShop();
                TestListSubShops();
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub TestAssignSubShop {
    my $MasterShopRef = LoadObjectByPath( $ROOTPATH . '/MyTestProvider/MyTestMasterShop' );
    my $SubShopRef    = LoadObjectByPath( $ROOTPATH . '/MyTestProvider/MyTestSubShop1' );
    ok( !IsSubShopAssignedToMasterShop( $MasterShopRef, $SubShopRef ), 'subshop not assigned' );
    AssignSubShopToMasterShop( $MasterShopRef, $SubShopRef );
    ok( IsSubShopAssignedToMasterShop( $MasterShopRef, $SubShopRef ), 'subshop is assigned' );
    dies_ok( sub { AssignSubShopToMasterShop( $MasterShopRef, $SubShopRef ) }, 'duplicate assignment dies' );
    UnassignSubShopFromMasterShop($SubShopRef);
    ok( !IsSubShopAssignedToMasterShop( $MasterShopRef, $SubShopRef ), 'subshop again not assigned' );
}

sub TestListSubShops {
    my $MasterShopRef = LoadObjectByPath( $ROOTPATH . '/MyTestProvider/MyTestMasterShop' );

    foreach my $i ( 1 .. 3 ) {
        my $SubShopRef = LoadObjectByPath( $ROOTPATH . '/MyTestProvider/MyTestSubShop' . $i );
        AssignSubShopToMasterShop( $MasterShopRef, $SubShopRef );
        is( $SubShopRef->get('MasterShopRef')->id, $MasterShopRef->id, 'mastershopref id matches' );
    }
    is( scalar @{ $MasterShopRef->get('SubShopRefs') }, 3, '3 assigned subshops' );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'MasterToSubShopRefMapper.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
