use base qw(Test::Class);
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::MultiStoreShipping::API::XML::ShippingLabelExportDriver;
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  JoinObjectPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use Test::More;
use Readonly;

use strict;

Readonly my $ROOTPATH => 'Test';

use Test::MockObject;

sub run_tests {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # initializsation
            ok( Init(), 'initialization' );
            eval {
                testImport();
                testExportShippingLabelsMocks();
                #testExportShippingLabels();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error - ' . $Error->code );
                diag( $Error->longMessage );

                # always cleanup
                ok( Cleanup(), 'cleanup' );
                Error($Error);
            }

            # cleanup
            ok( Cleanup(), 'cleanup' );
        }
    );
}

sub testImport {
    my $Shop = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestShopMultiStore' ] ) );
    isa_ok( $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );

    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/MultiStore');
    isa_ok( $ShippingFulfillmentMethod, 'DE_EPAGES::MultiStoreShipping::API::Object::ShippingFulfillmentMethodMultiStore' );

    my $Customer = $Shop->path('Customers/TestCustomer');
    isa_ok( $Customer, 'DE_EPAGES::Customer::API::Object::Customer' );

    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    isa_ok( $Order, 'DE_EPAGES::Order::API::Object::CustomerOrder' );

    cmp_ok( scalar @{ $Order->get('Documents') }, '>=', 1, 'number of orderdocuments' );

    my $ShippingLabel = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    isa_ok( $ShippingLabel, 'DE_EPAGES::MultiStoreShipping::API::Object::ShippingLabelMultiStore' );

    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderMultiStore');
    isa_ok( $Transaction, 'DE_EPAGES::ExternalShipping::API::Object::ShippingTransaction' );

    cmp_ok( scalar @{ $Transaction->get('ShippingLabels') }, '>=', 1, 'number of labels' );

    foreach my $ShippingLabel ( @{ $Transaction->get('ShippingLabels') } ) {
        isa_ok( $ShippingLabel, 'DE_EPAGES::MultiStoreShipping::API::Object::ShippingLabelMultiStore' );
    }
}

sub testExportShippingLabelsMocks {
    my $WebServiceClientFactory = Test::MockObject->new();
    my $CustomerService         = Test::MockObject->new();
    my $OrderService            = Test::MockObject->new();

    $CustomerService->mock( 'exists', sub { return 1;});
    $OrderService->mock( 'create', sub { return 1;});

    $WebServiceClientFactory->mock( 'isLoggingEnabled', sub { return 1; } );
    $WebServiceClientFactory->mock( 'OrderWebServiceClient', sub { return $OrderService; } );
    $WebServiceClientFactory->mock( 'CustomerWebServiceClient', sub { return $CustomerService; } );

    my $exportDriver = DE_EPAGES::MultiStoreShipping::API::XML::ShippingLabelExportDriver->new($WebServiceClientFactory);
    my $Shop         = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestShopMultiStore' ] ) );
    my $Transaction  = $Shop->path('ShippingOrders/ShippingOrderMultiStore');
    $exportDriver->exportXML( undef, undef, $Transaction, 1 );

    my $issuedCounter = 1;
    foreach my $ShippingLabel ( @{ $Transaction->get('ShippingLabels') } ) {
        cmp_ok( $ShippingLabel->get('Issued'), '==', 1, 'shipping label ' . $issuedCounter++ . ' is issued' );
    }
}


sub testExportShippingLabels {
    my $exportDriver = DE_EPAGES::MultiStoreShipping::API::XML::ShippingLabelExportDriver->new();
    my $Shop         = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestShopMultiStore' ] ) );
    my $Transaction  = $Shop->path('ShippingOrders/ShippingOrderMultiStore');

    $exportDriver->exportXML( undef, undef, $Transaction, 1 );

    my $issuedCounter = 1;
    foreach my $ShippingLabel ( @{ $Transaction->get('ShippingLabels') } ) {
        cmp_ok( $ShippingLabel->get('Issued'), '==', 1, 'shipping label ' . $issuedCounter++ . ' is issued' );
    }
}


sub Main {
    run_tests;
    done_testing;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
