use strict;
use Test::More tests => 63;
use DE_EPAGES::Core::API::File qw( CreateDirectory DeleteDirectory ExistsDirectory );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::Factory qw( DeleteObject ExistsObject ExistsObjectByPath LoadObjectByPath LoadRootObject LoadClassByAlias );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite GetCurrentDBHandle );

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::DefaultAttribute';
my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();

    # create test classes
    ok( Init(), "initialization" );

    my $ParentObject = LoadObjectByPath( $ROOTPATH );
    my $Object = $ParentObject->insertChild( 'Object', {
        Alias => 'MyObject',
    } );
    ok( $Object->exists, "InsertObject" );

    # uniqueChildAlias
    ok( $ParentObject->uniqueChildAlias('MyObject') eq 'MyObject1', 'uniqueChildAlias' );

    # object id
    ok( $Object->id > $ParentObject->id, "id is incremented" );
    ok( $Object->id == $Object->get( 'ID' ), "get('ID')" );

    # inherit
    ok( $Object->get('Inherit'), "Inherit" );

    # alias
    ok( $Object->alias eq 'MyObject', "Alias" );

    # guid
    ok( 36 == length( $Object->get('GUID') ) , "length(GUID)" );

    # children
    ok( 0 == scalar @{ $Object->get('ChildObjects') }, "attribute ChildObjects" );

    # test children cache
    my $TestCacheObject = $Object->insertChild('Object', {'Alias'=>'TestCache'});
    ok ($Object->existsChild('TestCache'), 'object child exists');
    ok (! $ParentObject->existsChild('TestCache'), 'parent child doesnt exists');
    $TestCacheObject->set({'Parent'=>$ParentObject});
    ok (! $Object->existsChild('TestCache'), 'object child doesnt exists');
    ok ($ParentObject->existsChild('TestCache'), 'parent child exists');
    $TestCacheObject->set({'Alias'=>'NewAlias'});
    ok (! $ParentObject->existsChild('TestCache'), 'changed alias doesnt exists');
    ok ($ParentObject->existsChild('NewAlias'), 'new alias exists');

    # parent path
    ok( $Object->pathString eq "$ROOTPATH/MyObject", "pathString" );
    ok( $Object->get('Path') eq "$ROOTPATH/MyObject", "attribute Path 1" );
    ok( 3 == scalar @{ $Object->get('ParentObjects') }, "attribute ParentObjects" );
    ok( $Object->get('ParentObjects')->[0]->id == LoadRootObject()->id, "root object" );
    ok( $Object->get('ParentObjects')->[-1]->id == $Object->id, "this object" );
    ok( $Object->get('ParentObjects')->[-2]->id == $Object->parent->id, "parent object" );

    ok( $Object->get('Path') eq "$ROOTPATH/MyObject", "attribute ObjectPath 2" );
    ok( ref $Object->get('AttributeValues') eq "ARRAY", "get AttributeValues" );

    # filePath
    my $StoreName = GetCurrentDBHandle()->storeName;
    my $FilePath = $Object->get('FilePath');
    ok( 39 == length $FilePath, 'FilePath' );

    my $PublicPath = $Object->get('PublicPath');
    CreateDirectory( $PublicPath );
    ok( $PublicPath eq "$ENV{EPAGES_WEBROOT}/$StoreName/$FilePath", 'PublicPath' );

    my $PrivatePath = $Object->get('PrivatePath');
    ok( $PrivatePath eq "$ENV{EPAGES_STORES}/$StoreName/$FilePath", 'PrivatePath' );
    {
        use utf8;
        $Object->set({ Alias => 'A/?ü"' });
        my $path = "$ROOTPATH/\"A/?ü\"\"\"";
        ok( $Object->get('Path') eq $path, 'Path special' );
        ok (ExistsObjectByPath($path), 'object with utf8 alias exists');
        ok ($Object->id == LoadObjectByPath($path)->id, 'object with utf8 is correct one');
    }

    # rebuild FilePathPart (if directory exists)
    DeleteDirectory( $PublicPath ); # not renamed automatically when FilePathPart is changed
    CreateDirectory( "$ENV{EPAGES_WEBROOT}/$StoreName$ROOTPATH/A_002F__003F__00FC__0022_" );
    $Object->set({ FilePathPart => 'rebuild' });
    ok( '/'. $Object->get('FilePath') eq "$ROOTPATH/A_002F__003F__00FC__0022_", 'FilePath special' );

    $PublicPath = $Object->get('PublicPath');
    ok( $PublicPath eq "$ENV{EPAGES_WEBROOT}/$StoreName$ROOTPATH/A_002F__003F__00FC__0022_", 'PublicPath special' );
    ok( $Object->get('PrivatePath') eq "$ENV{EPAGES_STORES}/$StoreName$ROOTPATH/A_002F__003F__00FC__0022_", 'PrivatePath special' );

    # change alias
    my $OldPublicPath = $Object->get('PublicPath');
    $Object->set({ Alias => 'MyObject' });
    ok( $OldPublicPath eq $Object->get('PublicPath'), "PublicPath has not changed" );
    ok( ExistsDirectory( $PublicPath ), "Directory still exists" );

    # attributes
    my $FalseType = undef;
    foreach my $Attribute ( @{ $Object->class->attributes } ) {
        my $Value = $Object->get($Attribute->alias);
        if( $Attribute->get('IsArray') ) {
            $FalseType = $Attribute->alias unless ref $Value eq 'ARRAY';
        }
        else {
            $FalseType = $Attribute->alias if ref $Value eq 'ARRAY';
        }
    }
    ok((not defined $FalseType), "non/is array " . (defined $FalseType ? $FalseType : ''));

    # AttributeValues
    my $ahValues = $Object->get('AttributeValues');
    my $aAttributes = $Object->get('Class')->get('Attributes');
    ok( scalar @$ahValues == scalar @$aAttributes, "AttributeValues" );

    # Child hash
    my $hChildObjects = $ParentObject->get('Child');
    my $aChildObjects = $ParentObject->children;
    ok( scalar keys %$hChildObjects == @$aChildObjects, "Child hash, number of keys" );
    ok( $hChildObjects->{'MyObject'}->{'ObjectID'} == $Object->id, "Child hash, object id" );

    # delete the object
    my $ObjectID = $Object->id;
    my $ObjectPath = $Object->pathString;
    ok( ExistsObjectByPath( $ObjectPath ), "before DeleteObject 1" );
    ok( ExistsObject( $ObjectID ), "before DeleteObject 2" );
    DeleteObject( $ObjectID );
    ok( !ExistsObjectByPath( $ObjectPath ), "after DeleteObject 1" );
    ok( !ExistsObject( $ObjectID ), "after DeleteObject 1" );
    DeleteDirectory( $PublicPath ) if defined $Object->get('FilePathPart');
    ok( !ExistsDirectory( $PublicPath ), "Directory has been deleted too" );

    # change object class
    my $ClassObject = LoadClassByAlias('Object');
    ok( $ClassObject->alias eq 'Object', "LoadClassByAlias Object" );
    $Object = $ParentObject->insertChild( $ClassObject, { Alias => 'MyObject' } );
    ok( $Object->get('Class')->id == $ClassObject->id, "Class before" );

    # System object
    my $System = LoadRootObject();
    ok( !defined $System->parent, "System has no parent" );
    ok( $System->get('Path') eq '/', "System path" );
    ok( $System->instanceOf('System'), "System object is from class System" );

    my $path = "$ROOTPATH/";
    if(ExistsObjectByPath($path)) {
        ok ( LoadObjectByPath($path)->id != LoadObjectByPath($ROOTPATH)->id, "$ROOTPATH/ not equal $ROOTPATH");
    } else {
        ok ( 1, $path . ' not exists');
    }

    $System = LoadRootObject();
    my $Classes = LoadObjectByPath('/Classes');
    my $ObjectClass = LoadObjectByPath('/Classes/Object');
    ok( !$Classes->isChildOf($ObjectClass), "Classes is not a child of ObjectClass" );
    ok( $Classes->isParentOf($ObjectClass), "Classes is a parent of ObjectClass" );
    ok( $ObjectClass->isChildOf($Classes), "ObjectClass is a child of Classes" );
    ok( !$ObjectClass->isParentOf($Classes), "ObjectClass is not a parent of Classes" );
    ok( $ObjectClass->isChildOf($System), "ObjectClass is a child of System" );
    ok( !$ObjectClass->isParentOf($System), "ObjectClass is not a parent of System" );

    is($System->relativePath($Classes), 'Classes', 'Classes relativePath');
    is($System->relativePath($ObjectClass), 'Classes/Object', 'Classes/Object relativePath');
    is($Classes->relativePath($ObjectClass), 'Object', 'Object relativePath');
    my $System2 = $Classes->path('..');
    is($System2->alias, 'System', 'system loaded by relative path');
    is($Classes->relativePath($System), '..', 'System relativePath from Classes');
    is($ObjectClass->relativePath($System), '../..', 'System relativePath from Object');

    my $OtherSite = $ParentObject->insertChild( 'Site', { 'Alias' => 'OtherSite', });
    is($System->relativePath($OtherSite), $OtherSite->pathString, 'relative path of objects from different sites.');

    ok( Cleanup(), 'Cleanup' );
    DisconnectSite();
}

sub Cleanup {
    if( ExistsObjectByPath( $ROOTPATH ) ) {
        LoadObjectByPath( $ROOTPATH )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();

    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
